<?php
/**
 * Librera para que aplicaciones PHP/GVHidra puedan acceder al Web Service de Autenticacion (autenticacinArangi)
 *
 * @package		WSClientAutentica
 * @author 		David Pascual <pascual_dav@externosgva.es>
 * @version		Release: @package_version@
 * @copyright 	Copyright (C) 2015 Generalitat Valenciana.
 * @license 	http://opensource.org/licenses/gpl-2.0.php GNU Public License v2.0
 * @link 		https://gvhidra.gva.es/redmine/projects/wscomun
 */
use WSCOMUN\WSComunFClient;

class WSClientAutentica extends WSComunFClient
{

	/**
	 * Permite autenticar va User/Pass de HDFI. (Subpaquete Autenticacion)
	 *
	 * Este mtodo realiza la autenticacin de un usuario a travs de usuario y password HDFI
	 * retornando un token generado en servidor en caso de xito.
	 * El WS pertenece al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$user	Usuario (Login HDFI)
	 * @param  	string	$pass	Password
	 * @return 	string	Cadena aleatorio (token) generado en el servidor
	 * @access 	public
	 */
	public function autenticaUsuarioWS ($user, $pass)
	{
		try
		{
			$this->__clienteOn(self::KEYCLIENTE_AUTENTICA, true);
			$clienteWS = $this->v_clienteWS[self::KEYCLIENTE_AUTENTICA];


			$autenticaUsuarioWSRequest = new ArrayObject();
			$autenticaUsuarioWSRequest->append(new SoapVar($user,XSD_STRING,null,null,'usuarioLDAP', self::$NAMESPACE_GVA));
			$autenticaUsuarioWSRequest->append(new SoapVar($pass,XSD_STRING,null,null,'pwdLDAP', self::$NAMESPACE_GVA));
			$miSoapRQ = new SoapVar($autenticaUsuarioWSRequest, SOAP_ENC_OBJECT,null,null,'autenticaUsuarioWSRequest', self::$NAMESPACE_GVA);
			$token = $clienteWS->autenticaUsuarioWS($miSoapRQ);

			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
			}

			if (is_object($token))
				if (property_exists($token, 'token'))
					if (!empty($token->token))
						return $token;

						if ($this->getDebugMode())
						{
							$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco');
						}

						return null;
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
				throw $e;
			}
			else throw $e;
		}
	}//autenticaUsuarioWS



	/**
	 * Permite autenticar va User/Pass de LDAP. (Subpaquete Autenticacion)
	 *
	 * Este mtodo realiza la autenticacin de un usuario a travs de usuario y password HDFI
	 * retornando un token generado en servidor en caso de xito.
	 * El WS pertenece al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$user	Usuario (LDAP)
	 * @param  	string	$pass	Password (LDAP)
	 * @return 	string	Cadena aleatorio (token) generado en el servidor o null si error
	 * @access 	public
	 */
	public function autenticaUsuarioLDAPWS ($user, $pass)
	{
		try
		{
			$this->__clienteOn(self::KEYCLIENTE_AUTENTICA, true);
			$clienteWS = $this->v_clienteWS[self::KEYCLIENTE_AUTENTICA];

			$autenticaUsuarioLDAPWSRequest = new ArrayObject();
			$autenticaUsuarioLDAPWSRequest->append(new SoapVar($user,XSD_STRING,null,null,'usuarioLDAP', self::$NAMESPACE_GVA));
			$autenticaUsuarioLDAPWSRequest->append(new SoapVar($pass,XSD_STRING,null,null,'pwdLDAP', self::$NAMESPACE_GVA));
			$miSoapRQ = new SoapVar($autenticaUsuarioLDAPWSRequest, SOAP_ENC_OBJECT,null,null,'autenticaUsuarioLDAPWSRequest', self::$NAMESPACE_GVA);
			$token = $clienteWS->autenticaUsuarioLDAPWS($miSoapRQ);

			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
			}

			if (is_object($token))
				if (property_exists($token, 'token'))
					if (!empty($token->token))
						return $token;

						if ($this->getDebugMode())
						{
							$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco');
						}

						return null;
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
				throw $e;
			}
			else throw $e;
		}
	}//Fin autenticaUsuarioLDAPWS


	/**
	 * Permite autenticar a travs de una firma realizada en cliente va certificado digital. (Subpaquete Autenticacion)
	 *
	 * Este mtodo realiza la autenticacin de un usuario a travs de una firma digital,
	 * retornando un token generado en servidor en caso de xito.
	 * El WS pertenece al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$firma	Firma realizada en cliente y codificada como cadena en base 64
	 * @param  	string	$codigoEntidad	Cdigo de la entidad que representa (opciona)
	 * @return 	string	Cadena aleatorio (token) generado en el servidor
	 * @access 	public
	 */
	public function autenticaConFirmaWS ($firmaB64, $codigoEntidad = null)
	{
		try
		{
			$this->__clienteOn(self::KEYCLIENTE_AUTENTICA);
			$clienteWS = $this->v_clienteWS[self::KEYCLIENTE_AUTENTICA];
			$autenticaConFirmaWSRequest = new ArrayObject();
			$autenticaConFirmaWSRequest->append(new SoapVar($firmaB64, XSD_STRING,null,null,'firma', self::$NAMESPACE_GVA));
			$autenticaConFirmaWSRequest->append(new SoapVar($codigoEntidad,XSD_STRING,null,null,'codigoEntidad', self::$NAMESPACE_GVA));
			$miSoapRQ = new SoapVar($autenticaConFirmaWSRequest, SOAP_ENC_OBJECT,null,null,'autenticaConFirmaWSRequest', self::$NAMESPACE_GVA);
			$token = $clienteWS->autenticaConFirmaWS($miSoapRQ);
			return ($token);
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
				throw $e;
			}
			else throw $e;
		}
	}//Fin autenticaConFirmaWS



	/**
	 * Devuelve la informacin del usuario autenticado relacionado con el token de entrada. (Subpaquete Autenticacion)
	 *
	 * Este mtodo obtiene informacin del ususario autenticado que gener el token de entrada.
	 * La informacin se presenta en un vector asociativo, cuyas claves dependen del si el token
	 * fue generado por un usuario autenticado va certificado o va user/pass.
	 *
	 * Si el token es de un usuario autenticado va CERTIFICADO, las claves son:
	 * [TipoAuth, SerialNumber, SubjectDN, IssuerDN, nombre, apellido1, apellido2, nif, cif, razonSocial, oidCert, tipoCert, email]
	 *
	 * Si el token es de un usuario autenticado va User/pass, las claves son:
	 * [TipoAuth, idHDFI, nombre, apellido1, apellido2, nif, cif, razonSocial]
	 *
	 * El WS pertenece al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$token	Cadena aleatoria asociada a un ususario autenticado
	 * @return 	array	array	Vector asociativo con informacin (claves) variable segun origern del token de entrada
	 * @access 	public
	 */
	public function getInformacionWS ($token)
	{
		if (empty($token))
		{
			if ($this->getDebugMode())
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco');

				throw new Exception (
						__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.
						'. Token vaco'
						);
		}

		try
		{
			$this->__clienteOn(self::KEYCLIENTE_AUTENTICA);
			$clienteWS = $this->v_clienteWS[self::KEYCLIENTE_AUTENTICA];
			$resultado = $clienteWS->getInformacionWS($token);
			return $resultado;
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
			}
			throw $e;
		}
	}//Fin getInformacionWS


	/**
	 * Devuelve informacin del usuario que se pase como parmetro de entrada (si el pass es correcto) (Mtodo de la clase. Subpaquete Autenticacion)
	 *
	 * Este mtodo no corresponde con ninguna invocacin directa a un WS, sino que es el resultado
	 * de dos llamadas independientes
	 *
	 *
	 * Los WS invocados pertenecen al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$user	Usuario (LDAP)
	 * @param  	string	$pass	Password (LDAP)
	 * @return 	mixed	$resultado	Vector asociativo con informacin del ususario o null en caso de error
	 * @access 	public
	 */
	public function getInformacionLDAP ($user, $pass)
	{
		$resultado = null;
		try //Autenticacin
		{
			$token = $this->autenticaUsuarioLDAPWS($user, $pass);

			if (!is_object($token))//Si no es un objeto
			{
				if ($this->getDebugMode())
				{
					$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. No es objeto');
				}
				return null;
			}
			else
			{
				if (!property_exists($token, 'token'))//No existe la property
				{
					if ($this->getDebugMode())
					{
						$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. No hay propiedad');
					}
					return null;
				}
				else
				{
					if (empty($token->token)) //La propiedad no tiene valor
					{
						if ($this->getDebugMode())
						{
							$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. Propiedad vaca');
						}
						return null;
					}
				}
			}
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				throw $e;
		}

		try //Informacin
		{
			$resultado = $this->getInformacionWS($token);
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				throw $e;
		}
		return $resultado;
	}//Fin getInformacionLDAP


	/**
	 * Devuelve informacin del usuario que se pase como parmetro de entrada (si el pass es correcto) (Mtodo de la clase. Subpaquete Autenticacion)
	 *
	 * Este mtodo no corresponde con ninguna invocacin directa a un WS, sino que es el resultado
	 * de dos llamadas independientes. Una a la autenticacin HDFI y otra a la obtencin de informacin
	 *
	 *
	 * Los WS invocados pertenecen al servicio de de Autenticacion (autenticacinArangi v1.00).
	 * @subpackage 	Autenticacion
	 * @param  	string	$user	Usuario (LDAP)
	 * @param  	string	$pass	Password (LDAP)
	 * @return 	mixed	$resultado	Vector asociativo con informacin del ususario o null si error de autenticacin
	 * @access 	public
	 */
	public function getInformacionHDFI ($user, $pass)
	{
		$resultado = null;
		try //Autenticacin
		{
			$token = $this->autenticaUsuarioWS($user, $pass);

			if (!is_object($token))//Si no es un objeto
			{
				if ($this->getDebugMode())
				{
					$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. No es objeto');
				}
				return null;
			}
			else
			{
				if (!property_exists($token, 'token'))//No existe la property
				{
					if ($this->getDebugMode())
					{
						$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. No hay propiedad');
					}
					return null;
				}
				else
				{
					if (empty($token->token)) //La propiedad no tiene valor
					{
						if ($this->getDebugMode())
						{
							$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'.'. Token vaco. Propiedad vaca');
						}
						return null;
					}
				}
			}
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				throw $e;
		}

		try //Informacin
		{
			$resultado = $this->getInformacionWS($token);
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				throw $e;
		}
		return $resultado;
	}//Fin getInformacionHDFI

}

?>